SystemOrganization addCategory: #SWA014!

ServiceProvider subclass: #SWA014ServiceProvider
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWA014'!

!SWA014ServiceProvider class methodsFor: 'initialization' stamp: 'TZ 2/10/2017 19:35'!
initialize 
	ServiceRegistry buildProvider: self new! !

Object subclass: #SWA014Spawner
	instanceVariableNames: 'game playerBlob swaworld'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWA014'!

!SWA014Spawner commentStamp: 'MV 2/5/2017 21:09' prior: 0!
I am the Spawner. I spawn all the things you see. I observe Enemy, to spawn a new Enemy, if it gets eaten.
!

!SWA014Spawner class methodsFor: 'initialization' stamp: 'MV 2/5/2017 15:09'!
newWith: aGame
	
	^ SWA014Spawner new
		initializeWith: aGame;
		yourself.
	
	
	! !

!SWA014Spawner methodsFor: 'constants' stamp: 'Tc 2/3/2017 11:26'!
defaultMaxEnemySize

	^ 100.! !

!SWA014Spawner methodsFor: 'accessing' stamp: 'MV 2/10/2017 16:24'!
enemySpawnPosition
	|intersection relativePosition absolutePosition|
	
	intersection := self swaworld fullBoundsInWorld intersect: World fullBoundsInWorld.
	relativePosition := (self swaworld extent - intersection extent) atRandom.
	absolutePosition := self swaworld position + relativePosition.
	(absolutePosition > (0 asPoint) and: [absolutePosition < World extent]) ifTrue: [relativePosition := relativePosition + World extent].
	
	^ relativePosition.! !

!SWA014Spawner methodsFor: 'accessing' stamp: 'MV 2/4/2017 16:39'!
game
	
	^ game.! !

!SWA014Spawner methodsFor: 'accessing' stamp: 'MV 2/4/2017 16:39'!
game: aGame
	
	game := aGame.! !

!SWA014Spawner methodsFor: 'spawning' stamp: 'MV 2/4/2017 16:43'!
giveUpgradesToEnemy: enemy
	|upgradeNumberToEnable listOfKeys randomUpgradeKey|

	listOfKeys := self game upgradeDictionary keys asOrderedCollection.
	upgradeNumberToEnable := listOfKeys size min: self playerBlob level.
	
	1 to: upgradeNumberToEnable do: [:aNumber|
		randomUpgradeKey := listOfKeys atRandom.
		self game enable: randomUpgradeKey for: enemy.
		listOfKeys remove: randomUpgradeKey].! !

!SWA014Spawner methodsFor: 'constants' stamp: 'MV 2/4/2017 14:31'!
initialNumberOfEnemies

	^ 50.! !

!SWA014Spawner methodsFor: 'initialization' stamp: 'TZ 2/10/2017 21:11'!
initializeWith: aGame
	super initialize.
	
	self 
		game: aGame;
		spawnWorld;
		spawnUpgradeBar;
		spawnPlayerBlob;
		spawnInitialEnemies;
		openScoreWindow;
		yourself.! !

!SWA014Spawner methodsFor: 'spawning' stamp: 'MV 2/4/2017 16:43'!
openScoreWindow
	|newScore|
	
	newScore := SWA014Score new. 
	self game addMorph: newScore.
	self playerBlob addDependent: newScore.! !

!SWA014Spawner methodsFor: 'accessing' stamp: 'MV 2/4/2017 16:40'!
playerBlob

	^ playerBlob.! !

!SWA014Spawner methodsFor: 'accessing' stamp: 'MV 2/4/2017 16:41'!
playerBlob: aLivingThing

	playerBlob := aLivingThing.! !

!SWA014Spawner methodsFor: 'spawning' stamp: 'MV 2/10/2017 14:03'!
randomEnemySize
	|maxSize|
	
	"In a quater of all cases we spawn a realy big (as big as the playerBlob) guy"
	maxSize := (Random new isTrueIn: 1 outOf: 4) 
		ifTrue: [(self playerBlob bodysize) asInteger]
		ifFalse: [self defaultMaxEnemySize].
	
	^ maxSize atRandom.
! !

!SWA014Spawner methodsFor: 'spawning' stamp: 'MV 2/7/2017 10:49'!
spawnEnemy
	|enemy random|
	
	random := Random new.
	enemy := SWA014Enemy new.
	
	"While the PlayerBlob gets upgrades when he evolves, Enemies get upgrades depending on the level of the playerBlob"
	self giveUpgradesToEnemy: enemy.
	
	self playerBlob listOfLivingThings add: enemy.
	self swaworld addMorph: enemy.

	enemy
		addDependent: self;
		bodysize: self randomEnemySize;
		swaworld: self swaworld;
		"realCenter: (random nextInt: self swaworld width) @ (random nextInt: self swaworld height);"
		realCenter: self enemySpawnPosition;
		listOfLivingThings: self playerBlob listOfLivingThings;
		yourself.! !

!SWA014Spawner methodsFor: 'spawning' stamp: 'MV 2/4/2017 16:50'!
spawnInitialEnemies

	1 to: self initialNumberOfEnemies do: [:aNumber| self spawnEnemy]. ! !

!SWA014Spawner methodsFor: 'spawning' stamp: 'TZ 2/10/2017 21:12'!
spawnPlayerBlob
	|newPlayerBlob|
	
	newPlayerBlob := SWA014PlayerBlob new.
	
	newPlayerBlob
		bodysize: 60;
		swaworld: self swaworld;
		upgradeBar: self game upgradeBar;
		realCenter: self swaworld extent / 2;
		addDependent: self game;
		yourself.
	
	self swaworld addMorph: newPlayerBlob.
	self playerBlob: newPlayerBlob.
	self game playerBlob: newPlayerBlob.
	self playerBlob listOfLivingThings add: newPlayerBlob.! !

!SWA014Spawner methodsFor: 'spawning' stamp: 'MV 2/4/2017 16:55'!
spawnUpgradeBar
	|aBar|
	
	aBar := SWA014UpgradeBar new position: self upgradeBarPosition.
	
	self game
		addMorph: aBar;
		upgradeBar: aBar;
		yourself.
	! !

!SWA014Spawner methodsFor: 'spawning' stamp: 'MV 2/5/2017 15:37'!
spawnWorld
	
	self swaworld: SWA014World new.
	self game addMorph: self swaworld.
	self swaworld
		realCenter: self game center;
		yourself.
	self swaworld image center: self game center.! !

!SWA014Spawner methodsFor: 'accessing' stamp: 'MV 2/4/2017 16:49'!
swaworld

	^ swaworld.! !

!SWA014Spawner methodsFor: 'accessing' stamp: 'MV 2/4/2017 16:49'!
swaworld: aWorld

	swaworld := aWorld.! !

!SWA014Spawner methodsFor: 'accessing' stamp: 'MV 2/5/2017 13:01'!
update: aMessage
	
	self spawnEnemy.! !

!SWA014Spawner methodsFor: 'constants' stamp: 'MV 2/4/2017 16:39'!
upgradeBarPosition

	^ (self game bounds right - 125) @ (self game bounds top + 50).! !

CircleMorph subclass: #SWA014LivingThing
	instanceVariableNames: 'bodysize speed speedSaver direction maxAllowedSize minAllowedSize minAllowedSpeed maxAllowedSpeed realCenter upgradeList swaworld listOfLivingThings'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWA014'!

!SWA014LivingThing commentStamp: 'MV 2/5/2017 17:10' prior: 0!
I am a LivingThing. I am the superclass of all moving objects participating in the game. I implement their logic for movement, collisions etc.!

SWA014LivingThing subclass: #SWA014Enemy
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWA014'!

!SWA014Enemy commentStamp: 'MV 2/5/2017 17:12' prior: 0!
I am an Enemy. I will try to kill you and other LivingThings. You could eat me to grow. :) I inherit from LivingThing, where most of my logic is described.!

!SWA014Enemy methodsFor: 'movement' stamp: 'MV 2/5/2017 21:37'!
calculateDirection: target
	"Enemys should follow the nearest object, which could be eaten by them. Also they should run away, if there is a bigger LivingThing (Player or Enemy) nearby"
	|direction|
	
	direction := super calculateDirection: target realCenter.
	^ self < target
		ifTrue: [direction negated]
		ifFalse: [direction].
	! !

!SWA014Enemy methodsFor: 'collision handling' stamp: 'MV 2/5/2017 13:13'!
getEaten

	self changed: self. "The Spawner should spawn an Enemy as a replacement"
	self delete.! !

!SWA014Enemy methodsFor: 'stepping' stamp: 'TZ 2/2/2017 20:29'!
step

	self 
		direction: (self calculateDirection: self nearestLivingThing);
		moveAutomatically;
		yourself. 
	
	self upgradeList do: [:aUpgrade| aUpgrade activateIfNotCooling].
	
	super step.! !

!SWA014LivingThing methodsFor: 'body transformation' stamp: 'TZ 2/2/2017 19:09'!
< aLivingThing

	^ self bodysize < aLivingThing bodysize.! !

!SWA014LivingThing methodsFor: 'body transformation' stamp: 'TZ 2/2/2017 19:09'!
> aLivingThing

	^ self bodysize > aLivingThing bodysize.! !

!SWA014LivingThing methodsFor: 'body transformation' stamp: 'TZ 2/2/2017 19:09'!
addMassOf: aLivingThing

	self bodysize: 2 * ((self squaredRadius + aLivingThing squaredRadius) sqrt).! !

!SWA014LivingThing methodsFor: 'constants' stamp: 'TZ 2/2/2017 19:50'!
bigNumber

	^ SmallInteger maxVal.! !

!SWA014LivingThing methodsFor: 'body transformation' stamp: 'TZ 2/2/2017 19:09'!
bodysize

	^ bodysize.! !

!SWA014LivingThing methodsFor: 'body transformation' stamp: 'MV 2/5/2017 15:20'!
bodysize: aSize
	
	bodysize := self maxAllowedSize min: (self minAllowedSize max: aSize).
! !

!SWA014LivingThing methodsFor: 'constants' stamp: 'TZ 2/10/2017 18:39'!
borderThickness
	
	^ 3.! !

!SWA014LivingThing methodsFor: 'movement' stamp: 'TZ 2/2/2017 19:17'!
calculateDirection: target

	^ self vectorToPoint: target.! !

!SWA014LivingThing methodsFor: 'movement' stamp: 'MV 2/5/2017 15:25'!
calculateRealCenterFor: aPosition
	"Converts a absolute position to a relative position like realCenter"
	
	^ aPosition - self swaworld realCenter + self swaworld radius asPoint.! !

!SWA014LivingThing methodsFor: 'collision handling' stamp: 'MV 2/5/2017 21:20'!
checkForBorderCollisions
	 
		self realCenter: (self realCenter x min: (self swaworld width - self radius)) @ (self realCenter y min: (self swaworld height - self radius)).
		self realCenter: (self realCenter x max: self radius) @ (self realCenter y max: self radius).! !

!SWA014LivingThing methodsFor: 'collision handling' stamp: 'MV 2/4/2017 18:56'!
checkForCollisions

	self checkForCollisionsWithLivingThings.
	self checkForBorderCollisions.! !

!SWA014LivingThing methodsFor: 'collision handling' stamp: 'MV 2/4/2017 17:19'!
checkForCollisionsWithLivingThings

	self listOfLivingThings do: [:aLivingThing|
		(self checkForOverlapWith: aLivingThing) ifTrue: [
			aLivingThing < self 
				ifTrue: [
					aLivingThing goBehind.
					(self squaredDistanceTo: aLivingThing realCenter) < self squaredRadius ifTrue: [
						self eat: aLivingThing]]
				ifFalse: [aLivingThing comeToFront].
			self swaworld image goBehind]] without: self.! !

!SWA014LivingThing methodsFor: 'collision handling' stamp: 'MV 2/10/2017 15:15'!
checkForOverlapWith: enemy

	^ self bottom > enemy top and: 
	    [self top < enemy bottom and:
	    [self left < enemy right and:
	    [self right > enemy left]]].
! !

!SWA014LivingThing methodsFor: 'constants' stamp: 'MV 2/10/2017 17:20'!
defaultDirection
	
	^ 0 asPoint.! !

!SWA014LivingThing methodsFor: 'constants' stamp: 'MV 2/10/2017 17:14'!
defaultSpeed
	
	^ 2.! !

!SWA014LivingThing methodsFor: 'movement' stamp: 'MV 2/10/2017 17:20'!
direction

	^ direction ifNil: [self defaultDirection].! !

!SWA014LivingThing methodsFor: 'movement' stamp: 'TZ 2/2/2017 19:32'!
direction: aDirection

	direction := aDirection.! !

!SWA014LivingThing methodsFor: 'collision handling' stamp: 'MV 2/5/2017 15:33'!
eat: aLivingThing

	self addMassOf: aLivingThing.
	self listOfLivingThings remove: aLivingThing.

	aLivingThing getEaten.! !

!SWA014LivingThing methodsFor: 'collision handling' stamp: 'TZ 2/2/2017 19:46'!
getEaten
	
	self delete.! !

!SWA014LivingThing methodsFor: 'initialization' stamp: 'TZ 2/10/2017 18:37'!
initialize
	super initialize.
	
	self 
		borderWidth: self borderThickness;
		resetBorderColor;
		listOfLivingThings: Set new;
		upgradeList: Dictionary new;
		yourself.! !

!SWA014LivingThing methodsFor: 'accessing' stamp: 'MV 2/10/2017 18:23'!
listOfLivingThings
	
	^ listOfLivingThings.! !

!SWA014LivingThing methodsFor: 'accessing' stamp: 'MV 2/4/2017 15:42'!
listOfLivingThings: aList
	
	listOfLivingThings := aList.! !

!SWA014LivingThing methodsFor: 'constants' stamp: 'MV 2/10/2017 17:07'!
maxAllowedSize

	^ 700.! !

!SWA014LivingThing methodsFor: 'constants' stamp: 'MV 2/10/2017 17:08'!
maxAllowedSpeed

	^ 100.! !

!SWA014LivingThing methodsFor: 'constants' stamp: 'MV 2/10/2017 17:08'!
minAllowedSize

	^ 15.! !

!SWA014LivingThing methodsFor: 'constants' stamp: 'MV 2/10/2017 17:09'!
minAllowedSpeed

	^ 0.! !

!SWA014LivingThing methodsFor: 'movement' stamp: 'TZ 2/3/2017 13:25'!
moveAutomatically

	self realCenter: self realCenter + (self speed * self direction).! !

!SWA014LivingThing methodsFor: 'vision' stamp: 'MV 2/5/2017 22:14'!
nearestLivingThing
	|nearestEnemyobject smallestDistance currentDistance |

	smallestDistance := self bigNumber.
	nearestEnemyobject := self.
		
	self listOfLivingThings do: [:aLivingThing| 
		currentDistance := self squaredDistanceTo: (aLivingThing realCenter).
		currentDistance  < smallestDistance ifTrue: [
			smallestDistance := currentDistance.
			nearestEnemyobject := aLivingThing]]
	without: self.

	^ nearestEnemyobject.! !

!SWA014LivingThing methodsFor: 'body transformation' stamp: 'TZ 2/2/2017 19:43'!
radius

	^ self bodysize / 2.! !

!SWA014LivingThing methodsFor: 'accessing' stamp: 'TZ 2/2/2017 19:42'!
realCenter

	^ realCenter.! !

!SWA014LivingThing methodsFor: 'accessing' stamp: 'MV 2/10/2017 17:04'!
realCenter: aPoint

	realCenter := aPoint.
	self refreshCenter.! !

!SWA014LivingThing methodsFor: 'upgrade management' stamp: 'TZ 2/10/2017 19:06'!
receiveUpgrade: anUpgrade

	self addMorph: anUpgrade.
	anUpgrade hide.! !

!SWA014LivingThing methodsFor: 'movement' stamp: 'TZ 2/2/2017 19:53'!
refreshCenter

	self center: self swaworld realCenter - (self swaworld extent / 2) + self realCenter.! !

!SWA014LivingThing methodsFor: 'body transformation' stamp: 'TZ 2/2/2017 19:44'!
refreshSize

	self extent: self bodysize asPoint.
	
	! !

!SWA014LivingThing methodsFor: 'body transformation' stamp: 'TZ 2/10/2017 18:41'!
resetBorderColor

	self borderColor: self color darker darker darker.! !

!SWA014LivingThing methodsFor: 'movement' stamp: 'TZ 2/2/2017 19:53'!
saveSpeed

	self speedSaver: self speed.! !

!SWA014LivingThing methodsFor: 'movement' stamp: 'MV 2/10/2017 17:13'!
speed

	^ speed ifNil: [self defaultSpeed].! !

!SWA014LivingThing methodsFor: 'movement' stamp: 'MV 2/5/2017 15:21'!
speed: aSpeed

	speed := self maxAllowedSpeed min: (self minAllowedSpeed max: aSpeed).
! !

!SWA014LivingThing methodsFor: 'movement' stamp: 'MV 2/10/2017 17:19'!
speedSaver

	^ speedSaver ifNil: [self speed].! !

!SWA014LivingThing methodsFor: 'movement' stamp: 'TZ 2/2/2017 19:55'!
speedSaver: aSpeed

	speedSaver := aSpeed.! !

!SWA014LivingThing methodsFor: 'vision' stamp: 'TZ 2/2/2017 19:33'!
squaredDistanceTo: enemy
	"Pythagoras' theorem without sqrt"
	
	^ (enemy x - self realCenter x) squared + (enemy y - self realCenter y) squared. ! !

!SWA014LivingThing methodsFor: 'body transformation' stamp: 'MV 2/5/2017 21:29'!
squaredRadius
	"The radius gets squared, so when we compare with him we don't need to extract the root. (Phytagoras)"
	
	^ self radius squared. ! !

!SWA014LivingThing methodsFor: 'stepping' stamp: 'TZ 2/2/2017 19:35'!
step

	self
		checkForCollisions;
		refreshSize;
		yourself.! !

!SWA014LivingThing methodsFor: 'stepping' stamp: 'MV 2/5/2017 21:16'!
stepTime

	^ 15.! !

!SWA014LivingThing methodsFor: 'accessing' stamp: 'TZ 2/2/2017 19:42'!
swaworld

	^ swaworld.! !

!SWA014LivingThing methodsFor: 'accessing' stamp: 'TZ 2/2/2017 19:43'!
swaworld: aWorld
	
	swaworld := aWorld.! !

!SWA014LivingThing methodsFor: 'accessing' stamp: 'MV 2/10/2017 18:23'!
upgradeList

	^ upgradeList.! !

!SWA014LivingThing methodsFor: 'accessing' stamp: 'TZ 2/2/2017 19:43'!
upgradeList: aList

	upgradeList := aList.! !

!SWA014LivingThing methodsFor: 'vision' stamp: 'MV 2/4/2017 18:37'!
vectorToPoint: aPoint
	
	^ (aPoint - self realCenter) normalizedWithoutZeroDivide.! !

SWA014LivingThing subclass: #SWA014PlayerBlob
	instanceVariableNames: 'level upgradeBar'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWA014'!

!SWA014PlayerBlob commentStamp: 'MV 2/5/2017 17:16' prior: 0!
I am the PlayerBlob. I will follow your orders and with me you can play the game. I eat Enemies until I evolve, so you can get Upgrades.!

!SWA014PlayerBlob methodsFor: 'body transformation' stamp: 'TZ 2/2/2017 19:01'!
checkForEvolution

	self bodysize > self evolutionSize ifTrue: [self evolve].! !

!SWA014PlayerBlob methodsFor: 'constants' stamp: 'MV 2/10/2017 17:14'!
defaultSpeed

	^ 3.! !

!SWA014PlayerBlob methodsFor: 'collision handling' stamp: 'TZ 2/10/2017 21:26'!
eat: aLivingThing
	super eat: aLivingThing.
	
	self
		changed: #playerGrew; "The Score should increase"
		checkForEvolution;
		yourself.! !

!SWA014PlayerBlob methodsFor: 'constants' stamp: 'MV 2/5/2017 16:00'!
evolutionSize
	
	^ 600.! !

!SWA014PlayerBlob methodsFor: 'body transformation' stamp: 'TZ 2/10/2017 21:29'!
evolve
	
	self
		resetAllSizes;
		level: self level + 1;
		changed: #playerEvolved; "The evolve menu should be shown"
		yourself.
	! !

!SWA014PlayerBlob methodsFor: 'collision handling' stamp: 'TZ 2/10/2017 21:28'!
getEaten

	self changed: #playerDied. "The game should be over"! !

!SWA014PlayerBlob methodsFor: 'initialization' stamp: 'TZ 2/10/2017 18:46'!
initialize
	super initialize.
	
	self
		color: Color red;
		resetBorderColor;
		yourself.! !

!SWA014PlayerBlob methodsFor: 'accessing' stamp: 'MV 2/10/2017 18:27'!
level

	^ level ifNil: [0].! !

!SWA014PlayerBlob methodsFor: 'accessing' stamp: 'TZ 2/2/2017 18:59'!
level: aNumber

	level := aNumber.! !

!SWA014PlayerBlob methodsFor: 'upgrade management' stamp: 'TZ 2/10/2017 21:16'!
receiveUpgrade: anUpgrade

	self upgradeBar add: anUpgrade.! !

!SWA014PlayerBlob methodsFor: 'movement' stamp: 'TZ 2/10/2017 17:20'!
refreshCenter

	self swaworld setViewCenter: self realCenter.
	super refreshCenter.! !

!SWA014PlayerBlob methodsFor: 'body transformation' stamp: 'MV 2/5/2017 15:55'!
resetAllSizes

	self listOfLivingThings do: [:aLivingThing| aLivingThing bodysize: aLivingThing bodysize / self shrinkFactor].! !

!SWA014PlayerBlob methodsFor: 'constants' stamp: 'MV 2/5/2017 15:55'!
shrinkFactor
	
	^ 10.! !

!SWA014PlayerBlob methodsFor: 'stepping' stamp: 'TZ 2/10/2017 17:36'!
step
	super step.
	
	self 
		direction: (self calculateDirection: self swaworld realMousePosition);
		moveAutomatically;
		yourself.! !

!SWA014PlayerBlob methodsFor: 'upgrade management' stamp: 'TZ 2/10/2017 21:19'!
upgradeBar

	^ upgradeBar.! !

!SWA014PlayerBlob methodsFor: 'upgrade management' stamp: 'TZ 2/10/2017 21:19'!
upgradeBar: anUpgradeBar

	upgradeBar := anUpgradeBar.! !

Morph subclass: #SWA014EvolveMenu
	instanceVariableNames: 'game'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWA014'!

!SWA014EvolveMenu commentStamp: 'MV 2/5/2017 21:08' prior: 0!
I am the EvolveMenu. I will be shown, if you gain enouth points. I will spawn a few EvolveOptions, where you can choose your upgrade. 
!

!SWA014EvolveMenu methodsFor: 'accessing' stamp: 'TZ 1/27/2017 18:38'!
game
	
	^ game.! !

!SWA014EvolveMenu methodsFor: 'accessing' stamp: 'TZ 1/27/2017 18:38'!
game: aGame

	game := aGame.! !

!SWA014EvolveMenu methodsFor: 'initialization' stamp: 'Tc 2/3/2017 11:25'!
initialize
	super initialize.
	
	self color: Color transparent.! !

!SWA014EvolveMenu methodsFor: 'constants' stamp: 'MV 2/5/2017 14:35'!
maximumNumberOfOptions

	^ 3.! !

!SWA014EvolveMenu methodsFor: 'accessing' stamp: 'MV 2/5/2017 15:04'!
positionFor: anOption from: anOptionList

	^ ((anOptionList indexOf: anOption) / (anOptionList size  + 1) * self width) @ (self center y).! !

!SWA014EvolveMenu methodsFor: 'accessing' stamp: 'MV 2/5/2017 21:35'!
showOptions: anOptionList
	|evolveOption|
	
	"We remove random elements from the list until there are only maximumNumberOfOptions left"
	1 to: (anOptionList size - self maximumNumberOfOptions) do: [:aNumber| anOptionList remove: anOptionList atRandom].
	
	anOptionList do: [:anOption|
		evolveOption := SWA014EvolveOption newWithUpgrade: anOption.
		evolveOption 
			game: self game;
			center: (self positionFor: anOption from: anOptionList);
			yourself.
		self addMorph: evolveOption].
	
	anOptionList isEmpty ifFalse: [self game pause].! !

Morph subclass: #SWA014EvolveOption
	instanceVariableNames: 'upgradeKey game'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWA014'!

!SWA014EvolveOption commentStamp: 'MV 2/5/2017 21:08' prior: 0!
I am an EvolveOption. I am one of the options, you are shown by the EvolveMenu. Here you can get the upgrade I display by clicking on the ChooseButton, which is created by me.
!

!SWA014EvolveOption class methodsFor: 'initialization' stamp: 'MV 2/5/2017 16:47'!
newWithUpgrade: anUpgradeKey

	^ self new
		initializeWith: anUpgradeKey;
		yourself.! !

!SWA014EvolveOption methodsFor: 'constants' stamp: 'MV 2/5/2017 22:02'!
chooseButtonHeightFactor

	^ 3 / 4.! !

!SWA014EvolveOption methodsFor: 'initialization' stamp: 'MV 2/10/2017 16:47'!
createChooseButton
	| chooseButton |
	
	chooseButton := SWA014Button onClick: [self getChosen].
	chooseButton center: (self center x) @ (self height * self chooseButtonHeightFactor).
	chooseButton label: 'Choose!!'.
	chooseButton color: Color red.
	self addMorph: chooseButton.! !

!SWA014EvolveOption methodsFor: 'initialization' stamp: 'MV 2/5/2017 22:03'!
createStringMorphForOption
	| stringMorph |

	stringMorph := StringMorph new.
	stringMorph acceptValue: self upgradeKey.	
	stringMorph center: (self center x) @ (self height * self stringMorphHeightFactor).
	self addMorph: stringMorph.! !

!SWA014EvolveOption methodsFor: 'constants' stamp: 'MV 2/10/2017 17:23'!
evolveOptionSize
	
	^ 150 @ 250.! !

!SWA014EvolveOption methodsFor: 'accessing' stamp: 'TZ 1/27/2017 16:30'!
game

	^ game.! !

!SWA014EvolveOption methodsFor: 'accessing' stamp: 'TZ 1/27/2017 16:30'!
game: aGame

	game := aGame.! !

!SWA014EvolveOption methodsFor: 'upgrade managment' stamp: 'MV 2/5/2017 22:11'!
getChosen

	self game enable: self upgradeKey for: self game playerBlob.
	self owner delete.
	self game continue.
	self game upgradesUnavailableToPlayerBlob remove: self upgradeKey.! !

!SWA014EvolveOption methodsFor: 'initialization' stamp: 'MV 2/10/2017 16:59'!
initializeWith: anUpgradeKey
	
	self 
		extent: self evolveOptionSize;
		color: Color white;
		upgradeKey: anUpgradeKey;
		createChooseButton;
		createStringMorphForOption;
		yourself.
! !

!SWA014EvolveOption methodsFor: 'constants' stamp: 'MV 2/5/2017 22:03'!
stringMorphHeightFactor

	^ 1 / 4.! !

!SWA014EvolveOption methodsFor: 'accessing' stamp: 'MV 2/5/2017 16:45'!
upgradeKey

	^ upgradeKey.! !

!SWA014EvolveOption methodsFor: 'accessing' stamp: 'MV 2/5/2017 16:45'!
upgradeKey: anUpgradeKey

	upgradeKey := anUpgradeKey.! !

Morph subclass: #SWA014Game
	instanceVariableNames: 'upgradeDictionary upgradesUnavailableToPlayerBlob playerBlob isPaused upgradeBar aPosition spawner'
	classVariableNames: 'Highscore'
	poolDictionaries: ''
	category: 'SWA014'!

!SWA014Game commentStamp: 'MV 2/5/2017 17:04' prior: 0!
I am the Game. I handle many events and react to them. For example I can enable an Upgrade for somebody or react if you press a Key on your Keyboard. At the begin of each game I will initialize some data structures.!

!SWA014Game methodsFor: 'game events' stamp: 'MV 2/4/2017 17:18'!
continue

	self playerBlob listOfLivingThings do: [:aLivingThing| aLivingThing speed: aLivingThing speedSaver].
	self isPaused: false.! !

!SWA014Game methodsFor: 'upgrade managment' stamp: 'TZ 2/10/2017 19:07'!
enable: anUpgradeKey for: aLivingThing
	|anUpgrade|
	
	anUpgrade := (self upgradeDictionary at: anUpgradeKey ifAbsent: [SWA014NullUpgrade]) new.
	anUpgrade upgradeOwner: aLivingThing.
	aLivingThing receiveUpgrade: anUpgrade.
	aLivingThing upgradeList add: (anUpgradeKey->anUpgrade).
	anUpgrade stopStepping.
! !

!SWA014Game methodsFor: 'game events' stamp: 'MV 2/4/2017 13:58'!
gameOver

	self delete.! !

!SWA014Game methodsFor: 'input events' stamp: 'MV 2/5/2017 21:26'!
handleKeystroke: anEvent
	|keyValue upgradeKey|
	
	
	
	keyValue := anEvent keyValue.
	
	upgradeKey := keyValue caseOf: {
		[49 "Key: 1"]->[#Trap].
		[50 "Key: 2"]->[#Speedboost].
		[51 "Key: 3"]->[#Magnet].
		[52 "Key: 4"]->[#Drain].
		[53 "Key: 5"]->[#Shockwave]} otherwise: [#Nothing].
	
	upgradeKey ~= #Nothing ifTrue: [
		(self playerBlob upgradeList at: upgradeKey ifAbsent: [SWA014NullUpgrade new]) activateIfNotCooling].
	
	keyValue = 120 ifTrue: [ "Key: x"
		self delete].	
	keyValue = 112 ifTrue: [ "Key: p"
		self isPaused 
			ifTrue: [self continue]
			ifFalse: [self pause]].	
! !

!SWA014Game methodsFor: 'input events' stamp: 'TZ 2/2/2017 17:19'!
handlesMouseOver: anEvent
	
	^ true.! !

!SWA014Game methodsFor: 'initialization' stamp: 'MV 2/10/2017 18:07'!
initialize
	super initialize.
	
	self 
		extent: World extent;
		clipSubmorphs: true;
					
		initializeTextures;
		initializeDictionary;
		initializeUpgradesUnavailableToPlayerBlob;
		initializeSpawner;
				
		openInWorld;
		yourself.
		
! !

!SWA014Game methodsFor: 'initialization' stamp: 'TZ 2/3/2017 16:10'!
initializeDictionary
	
	self upgradeDictionary: (IdentityDictionary newFrom: {
		#Trap->SWA014TrapUpgrade.
		#Speedboost->SWA014SpeedboostUpgrade.
		#Magnet->SWA014MagnetUpgrade.
		#Drain->SWA014DrainUpgrade.
		#Shockwave->SWA014ShockwaveUpgrade}).! !

!SWA014Game methodsFor: 'initialization' stamp: 'MV 2/5/2017 21:36'!
initializeSpawner

	"The Game doesn't need to know the Spawner, but someone needs to hold a reference on him so he isn't deleted by the garbage collector"
	self spawner: (SWA014Spawner newWith: self).! !

!SWA014Game methodsFor: 'initialization' stamp: 'TZ 2/3/2017 16:24'!
initializeTextures
	
	SWA014Trap loadTexture.! !

!SWA014Game methodsFor: 'initialization' stamp: 'TZ 1/27/2017 19:32'!
initializeUpgradesUnavailableToPlayerBlob

	self upgradesUnavailableToPlayerBlob: self upgradeDictionary keys asOrderedCollection.! !

!SWA014Game methodsFor: 'accessing' stamp: 'MV 2/10/2017 17:55'!
isPaused

	^ isPaused ifNil: [false].! !

!SWA014Game methodsFor: 'accessing' stamp: 'TZ 1/27/2017 16:42'!
isPaused: aBoolean

	isPaused := aBoolean.! !

!SWA014Game methodsFor: 'input events' stamp: 'TZ 2/2/2017 17:19'!
mouseEnter: anEvent

	anEvent hand newKeyboardFocus: self.! !

!SWA014Game methodsFor: 'input events' stamp: 'TZ 2/2/2017 17:19'!
mouseLeave: anEvent

	anEvent hand newKeyboardFocus: nil.! !

!SWA014Game methodsFor: 'game events' stamp: 'MV 2/4/2017 17:19'!
pause

	self playerBlob listOfLivingThings do: [:aLivingThing|
		aLivingThing
			saveSpeed;
			speed: 0].
	self isPaused: true.! !

!SWA014Game methodsFor: 'accessing' stamp: 'MV 2/4/2017 14:45'!
playerBlob
	
	^ playerBlob.! !

!SWA014Game methodsFor: 'accessing' stamp: 'MV 2/4/2017 14:45'!
playerBlob: aLivingThing
	
	playerBlob := aLivingThing.! !

!SWA014Game methodsFor: 'game events' stamp: 'MV 2/10/2017 17:57'!
showEvolveOptions
	|evolveMenu|

	evolveMenu := SWA014EvolveMenu new.
	evolveMenu
		game: self;
		extent: self extent;
		center: self center;
		showOptions: self upgradesUnavailableToPlayerBlob copy;
		yourself.
		
	self addMorph: evolveMenu.! !

!SWA014Game methodsFor: 'accessing' stamp: 'MV 2/5/2017 12:54'!
spawner

	^ spawner.! !

!SWA014Game methodsFor: 'accessing' stamp: 'MV 2/5/2017 12:53'!
spawner: aSpawner
	
	spawner := aSpawner.! !

!SWA014Game methodsFor: 'updating' stamp: 'TZ 2/10/2017 21:32'!
update: aKey

	aKey caseOf: {
		[#playerDied]->[self delete].
		[#playerEvolved]->[self showEvolveOptions]} otherwise: [].! !

!SWA014Game methodsFor: 'accessing' stamp: 'TZ 1/27/2017 16:42'!
upgradeBar

	^ upgradeBar.! !

!SWA014Game methodsFor: 'accessing' stamp: 'TZ 1/27/2017 16:42'!
upgradeBar: aBar

	upgradeBar := aBar.! !

!SWA014Game methodsFor: 'accessing' stamp: 'TZ 1/27/2017 16:42'!
upgradeDictionary

	^ upgradeDictionary.! !

!SWA014Game methodsFor: 'accessing' stamp: 'TZ 1/27/2017 16:42'!
upgradeDictionary: aDictionary

	upgradeDictionary := aDictionary.! !

!SWA014Game methodsFor: 'accessing' stamp: 'TZ 1/27/2017 16:50'!
upgradesUnavailableToPlayerBlob

	^ upgradesUnavailableToPlayerBlob.! !

!SWA014Game methodsFor: 'accessing' stamp: 'TZ 1/27/2017 16:49'!
upgradesUnavailableToPlayerBlob: aList

	upgradesUnavailableToPlayerBlob := aList.! !

Morph subclass: #SWA014Menu
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWA014'!

!SWA014Menu commentStamp: 'MV 2/5/2017 17:17' prior: 0!
I am the Menu. You can start or end the game here.
!

!SWA014Menu class methodsFor: 'initialization' stamp: 'MV 2/5/2017 16:48'!
open

	^ SWA014Menu new openInWorld.! !

!SWA014Menu methodsFor: 'initialization' stamp: 'MV 2/10/2017 16:48'!
createEndButton
	|button|

	button := SWA014Button onClick: [self delete].
	button 
		label: 'End Game';
		center: (self width / 2) @ ((13 / 25) * self height);
		yourself.
	
	^ button.! !

!SWA014Menu methodsFor: 'initialization' stamp: 'MV 2/10/2017 16:40'!
createImage
	
	^ ImageMorph new image: (Form fromFileNamed: 'Pattern.png').! !

!SWA014Menu methodsFor: 'initialization' stamp: 'MV 2/10/2017 17:58'!
createStartButton
	|button|
	
	button := SWA014Button onClick: [SWA014Game new].
	button 
		label: 'Start Game';
		center: (self width / 2) @ ((12 / 25) * self height);
		color: Color green;
		yourself.
	
	^ button.! !

!SWA014Menu methodsFor: 'initialization' stamp: 'MV 2/10/2017 16:40'!
initialize 	
	super initialize.
	
	self
		extent: World extent;
		addMorph: self createImage;
		addMorph: self createStartButton;
		addMorph: self createEndButton;
		clipSubmorphs: true;
		yourself.! !

Morph subclass: #SWA014UpgradeBar
	instanceVariableNames: 'upgradeCount'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWA014'!

!SWA014UpgradeBar commentStamp: 'MV 2/5/2017 21:09' prior: 0!
I am the UpgradeBar. I display, which upgrades you have and how you can use them.
!

!SWA014UpgradeBar methodsFor: 'accessing' stamp: 'MV 2/5/2017 16:24'!
add: anUpgrade

	anUpgrade
		center: (self center x) @ (self position y + self offset + (self upgradeCount * self spaceBetweenUpgrades));
		extent: self upgradeSize asPoint;
		yourself.
	
	self 
		increaseUpgradeCount;
		addMorph: anUpgrade;
		yourself.
	! !

!SWA014UpgradeBar methodsFor: 'accessing' stamp: 'MV 2/5/2017 16:24'!
increaseUpgradeCount
	
	^ self upgradeCount: self upgradeCount + 1.! !

!SWA014UpgradeBar methodsFor: 'initialization' stamp: 'MV 2/10/2017 19:23'!
initialize
	super initialize.
	
	self 
		extent: self upgradeBarSize;
		color: Color transparent;
		yourself.! !

!SWA014UpgradeBar methodsFor: 'constants' stamp: 'MV 2/5/2017 16:23'!
offset
	
	^ 50.! !

!SWA014UpgradeBar methodsFor: 'constants' stamp: 'MV 2/5/2017 16:22'!
spaceBetweenUpgrades

	^ 100.! !

!SWA014UpgradeBar methodsFor: 'constants' stamp: 'MV 2/10/2017 19:18'!
upgradeBarSize
	
	^ 50 @ 500.! !

!SWA014UpgradeBar methodsFor: 'accessing' stamp: 'MV 2/10/2017 19:23'!
upgradeCount

	^ upgradeCount ifNil: [0].! !

!SWA014UpgradeBar methodsFor: 'accessing' stamp: 'MV 2/5/2017 16:11'!
upgradeCount: aNumber

	upgradeCount := aNumber.! !

!SWA014UpgradeBar methodsFor: 'initialization' stamp: 'MV 2/5/2017 16:12'!
upgradeSize

	^ 75.! !

Morph subclass: #SWA014World
	instanceVariableNames: 'worldSize realCenter image'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWA014'!

!SWA014World commentStamp: 'MV 2/5/2017 21:02' prior: 0!
I am the World. I am the (next) big thing you can see. In me the game is played. All LivingThings are submorphs of me.!

!SWA014World methodsFor: 'constant' stamp: 'MV 2/5/2017 19:54'!
amountOfPixelsInPattern

	^ 144.! !

!SWA014World methodsFor: 'movement' stamp: 'TZ 2/10/2017 17:22'!
calculateRealCenterFor: aPosition
	"Converts a absolute position to a relative position like realCenter"
	
	^ aPosition - self realCenter + self radius asPoint.! !

!SWA014World methodsFor: 'initialization' stamp: 'MV 2/10/2017 19:25'!
createImage

	^ ImageMorph new image: (Form fromFileNamed: 'Pattern.png').! !

!SWA014World methodsFor: 'accessing' stamp: 'TZ 2/3/2017 12:52'!
image

	^ image.! !

!SWA014World methodsFor: 'accessing' stamp: 'TZ 2/3/2017 12:52'!
image: anImage

	image := anImage.! !

!SWA014World methodsFor: 'initialization' stamp: 'MV 2/10/2017 19:36'!
initialize
	super initialize.
		
	self 
		image: self createImage;
		addMorph: self image;
		extent: self worldSize asPoint;	
		center: self realCenter;
		yourself.
		! !

!SWA014World methodsFor: 'body transformation' stamp: 'MV 2/10/2017 19:39'!
radius

	^ self worldSize / 2.! !

!SWA014World methodsFor: 'accessing' stamp: 'MV 2/10/2017 19:29'!
realCenter
	"Unlinke realCenter in other Morphs this realCenter is not relative to SWA014World"
	
	^ realCenter ifNil:[0 asPoint].! !

!SWA014World methodsFor: 'accessing' stamp: 'TZ 2/10/2017 17:16'!
realCenter: aPoint

	realCenter := aPoint.
	self refreshCenter.! !

!SWA014World methodsFor: 'accessing' stamp: 'TZ 2/10/2017 17:37'!
realMousePosition

	^ self calculateRealCenterFor: World activeHand center.! !

!SWA014World methodsFor: 'movement' stamp: 'MV 2/5/2017 19:57'!
refreshCenter

	self center: self realCenter.
	self image position: (self position \\ self amountOfPixelsInPattern - self amountOfPixelsInPattern).! !

!SWA014World methodsFor: 'movement' stamp: 'TZ 2/10/2017 17:21'!
setViewCenter: aPoint

	self realCenter: self realCenter + (self calculateRealCenterFor: self owner center) - aPoint.! !

!SWA014World methodsFor: 'body transformation' stamp: 'MV 2/10/2017 19:26'!
worldSize

	^ worldSize ifNil: [3000].! !

!SWA014World methodsFor: 'body transformation' stamp: 'MV 2/4/2017 16:58'!
worldSize: aSize

	worldSize := aSize.! !

ImageMorph subclass: #SWA014Trap
	instanceVariableNames: 'scalingFactor preySpeed listOfLivingThings realCenter'
	classVariableNames: 'Texture'
	poolDictionaries: ''
	category: 'SWA014'!

!SWA014Trap commentStamp: 'MV 2/5/2017 21:09' prior: 0!
I am a Trap. If you have the TrapUpgrade, you can throw me. If i collide with a LivingThing I will stop and shrink it for a short duration.
!

!SWA014Trap class methodsFor: 'initialization' stamp: 'MV 2/10/2017 18:37'!
loadTexture

	self texture: (Form fromFileNamed: 'Trap.png').! !

!SWA014Trap class methodsFor: 'accessing' stamp: 'MV 2/10/2017 18:35'!
texture
	
	^ Texture.! !

!SWA014Trap class methodsFor: 'accessing' stamp: 'MV 2/10/2017 18:37'!
texture: aImage
	
	Texture := aImage.! !

!SWA014Trap methodsFor: 'constants' stamp: 'TZ 2/2/2017 20:43'!
ailmentDuration

	^ 5000.! !

!SWA014Trap methodsFor: 'collision handling' stamp: 'TZ 2/10/2017 17:06'!
checkForCollisionsWithLivingThings

	self listOfLivingThings do: [:aLivingThing|
		(aLivingThing squaredDistanceTo: self realCenter) < aLivingThing squaredRadius ifTrue: [
			self collideWith: aLivingThing]].! !

!SWA014Trap methodsFor: 'collision handling' stamp: 'TZ 2/2/2017 20:44'!
collideWith: aLivingThing

	self
		inflictAilmentTo: aLivingThing;
		futureDo: #removeAilmentOf: at: self ailmentDuration args: (Array with: aLivingThing);
		delete.! !

!SWA014Trap methodsFor: 'ailments' stamp: 'TZ 2/2/2017 20:38'!
inflictAilmentTo: aLivingThing

	aLivingThing bodysize: aLivingThing bodysize / self scalingFactor.
	aLivingThing saveSpeed.
	aLivingThing speed: 0.! !

!SWA014Trap methodsFor: 'initialization' stamp: 'MV 2/10/2017 18:39'!
initialize
	super initialize.
	
	self 
		image: SWA014Trap texture;
		extent: self trapSize asPoint;
		yourself.! !

!SWA014Trap methodsFor: 'accessing' stamp: 'MV 2/5/2017 13:31'!
listOfLivingThings

	^ listOfLivingThings.! !

!SWA014Trap methodsFor: 'accessing' stamp: 'MV 2/5/2017 13:31'!
listOfLivingThings: aList

	listOfLivingThings := aList.! !

!SWA014Trap methodsFor: 'accessing' stamp: 'TZ 2/10/2017 17:05'!
realCenter

	^ realCenter.! !

!SWA014Trap methodsFor: 'accessing' stamp: 'TZ 2/10/2017 17:05'!
realCenter: aPoint

	realCenter := aPoint.! !

!SWA014Trap methodsFor: 'ailments' stamp: 'TZ 2/2/2017 20:38'!
removeAilmentOf: aLivingThing

	aLivingThing speed: aLivingThing speedSaver.
	aLivingThing bodysize: aLivingThing bodysize * self scalingFactor.! !

!SWA014Trap methodsFor: 'accessing' stamp: 'MV 2/10/2017 18:39'!
scalingFactor

	^ scalingFactor  ifNil: [1.5].! !

!SWA014Trap methodsFor: 'accessing' stamp: 'TZ 2/2/2017 20:36'!
scalingFactor: aFactor

	scalingFactor := aFactor.! !

!SWA014Trap methodsFor: 'stepping' stamp: 'TZ 2/2/2017 20:45'!
step

	self checkForCollisionsWithLivingThings.! !

!SWA014Trap methodsFor: 'stepping' stamp: 'MV 2/5/2017 21:16'!
stepTime

	^ 15.! !

!SWA014Trap methodsFor: 'constants' stamp: 'MV 2/10/2017 18:39'!
trapSize

	^ 15.! !

ImageMorph subclass: #SWA014Upgrade
	instanceVariableNames: 'upgradeOwner coolDownMorph cooling cooldown'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWA014'!

!SWA014Upgrade commentStamp: 'MV 2/5/2017 20:50' prior: 0!
I am an Upgrade. I am the superclass of all upgrades and implement their basic logic, for example the cooldown.!

SWA014Upgrade subclass: #SWA014DrainUpgrade
	instanceVariableNames: 'drainTarget'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWA014'!

!SWA014DrainUpgrade commentStamp: 'MV 2/5/2017 21:09' prior: 0!
I am a DrainUpgrade. I can drain a LivingThing, so it looses its bodysize, while my upgradeOwner gets size.
!

!SWA014DrainUpgrade methodsFor: 'starting up' stamp: 'TZ 2/10/2017 19:10'!
activateUpgrade

	self drainTarget: self upgradeOwner nearestLivingThing.
	self drainTarget borderColor: Color lavender.
	self startStepping.
	self futureDo: #stopStepping at: self drainDuration args: #().
	self drainTarget futureDo: #resetBorderColor at: self drainDuration args: #().! !

!SWA014DrainUpgrade methodsFor: 'constants' stamp: 'TZ 2/2/2017 20:53'!
amountOfDrain

	^ 3.! !

!SWA014DrainUpgrade methodsFor: 'constants' stamp: 'TZ 2/2/2017 20:56'!
drainDuration

	^ 4000.! !

!SWA014DrainUpgrade methodsFor: 'accessing' stamp: 'TZ 2/2/2017 20:51'!
drainTarget

	^ drainTarget.! !

!SWA014DrainUpgrade methodsFor: 'accessing' stamp: 'TZ 2/2/2017 20:51'!
drainTarget: aLivingThing

	drainTarget := aLivingThing.! !

!SWA014DrainUpgrade methodsFor: 'constants' stamp: 'TZ 2/10/2017 19:43'!
imageFileName

	^ 'DrainUpgrade.png'.! !

!SWA014DrainUpgrade methodsFor: 'stepping' stamp: 'TZ 2/10/2017 19:10'!
step

	self drainTarget ifNotNil: [
		self drainTarget bodysize: (self drainTarget bodysize - self amountOfDrain).
		self upgradeOwner bodysize: (self upgradeOwner bodysize + self amountOfDrain)].! !

!SWA014DrainUpgrade methodsFor: 'stepping' stamp: 'TZ 2/2/2017 20:55'!
stepTime

	^ 400.
	! !

SWA014Upgrade subclass: #SWA014ForcefieldUpgrade
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWA014'!

!SWA014ForcefieldUpgrade methodsFor: 'starting up' stamp: 'TZ 2/10/2017 20:01'!
activateUpgrade

	self startStepping.
	self futureDo: #stopStepping at: self forcefieldDuration args: #().
! !

!SWA014ForcefieldUpgrade methodsFor: 'constants' stamp: 'TZ 2/10/2017 19:56'!
forcefieldDuration

	self SubclassResponsibility.! !

!SWA014ForcefieldUpgrade methodsFor: 'constants' stamp: 'TZ 2/10/2017 19:57'!
forcefieldEnergy

	self SubclassResponsibility.! !

!SWA014ForcefieldUpgrade methodsFor: 'stepping' stamp: 'TZ 2/10/2017 20:01'!
step
	|force vector|
	
	self upgradeOwner listOfLivingThings do: [:aLivingThing|
		vector := self upgradeOwner realCenter - aLivingThing realCenter.
		force := self forcefieldEnergy / (vector dist: 0 asPoint).
		aLivingThing realCenter: aLivingThing realCenter + (vector normalized * force)] without: self upgradeOwner.
! !

!SWA014ForcefieldUpgrade methodsFor: 'stepping' stamp: 'TZ 2/10/2017 19:58'!
stepTime

	^ 10.! !

SWA014ForcefieldUpgrade subclass: #SWA014MagnetUpgrade
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWA014'!

!SWA014MagnetUpgrade commentStamp: 'MV 2/5/2017 20:55' prior: 0!
I am a MagnetUpgrade. I make the LivingThings around my upgradeOwner move towords him.!

!SWA014MagnetUpgrade methodsFor: 'constants' stamp: 'MV 2/10/2017 20:14'!
forcefieldDuration

	^ 5000.! !

!SWA014MagnetUpgrade methodsFor: 'constants' stamp: 'TZ 2/10/2017 20:00'!
forcefieldEnergy
	
	^ 625.! !

!SWA014MagnetUpgrade methodsFor: 'constants' stamp: 'TZ 2/10/2017 19:44'!
imageFileName

	^ 'MagnetUpgrade.png'.! !

SWA014ForcefieldUpgrade subclass: #SWA014ShockwaveUpgrade
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWA014'!

!SWA014ShockwaveUpgrade commentStamp: 'MV 2/5/2017 20:57' prior: 0!
I am a ShockwaveUpgrade. I push away all LivingThings, which are nearby me.
!

!SWA014ShockwaveUpgrade methodsFor: 'constants' stamp: 'MV 2/10/2017 20:16'!
forcefieldDuration

	^ 150.! !

!SWA014ShockwaveUpgrade methodsFor: 'constants' stamp: 'TZ 2/10/2017 20:02'!
forcefieldEnergy

	^ -8000.! !

!SWA014ShockwaveUpgrade methodsFor: 'constants' stamp: 'TZ 2/10/2017 19:46'!
imageFileName
	
	^ 'ShockwaveUpgrade.png'.! !

SWA014Upgrade subclass: #SWA014NullUpgrade
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWA014'!

!SWA014NullUpgrade commentStamp: 'MV 2/5/2017 20:56' prior: 0!
I am a NullUpgrade. I intentionally do nothing. So you can use me, if you try to use an upgrade, which you don't got yet.
!

!SWA014NullUpgrade methodsFor: 'starting up' stamp: 'TZ 2/2/2017 20:57'!
activateUpgrade
	"This class should do nothing"! !

SWA014Upgrade subclass: #SWA014SpeedboostUpgrade
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWA014'!

!SWA014SpeedboostUpgrade commentStamp: 'MV 2/5/2017 20:57' prior: 0!
I am a SpeedboostUpgrade. Activate me to get increased speed for a short duration.
!

!SWA014SpeedboostUpgrade methodsFor: 'starting up' stamp: 'TZ 2/2/2017 21:12'!
activateUpgrade

	self upgradeOwner 
		speed: self upgradeOwner speed + self speedIncreasement;
		futureDo: #speed: at: self speedboostDuration args: (Array with: (self upgradeOwner speed - self speedIncreasement));
		yourself.! !

!SWA014SpeedboostUpgrade methodsFor: 'constants' stamp: 'TZ 2/10/2017 19:49'!
imageFileName

	^ 'SpeedboostUpgrade.png'.! !

!SWA014SpeedboostUpgrade methodsFor: 'constants' stamp: 'TZ 2/2/2017 21:10'!
speedIncreasement
	
	^ 11.! !

!SWA014SpeedboostUpgrade methodsFor: 'constants' stamp: 'TZ 2/2/2017 21:09'!
speedboostDuration

	^ 1000.! !

SWA014Upgrade subclass: #SWA014TrapUpgrade
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWA014'!

!SWA014TrapUpgrade commentStamp: 'MV 2/5/2017 20:58' prior: 0!
I am a TrapUpgrade. Activate me to throw a Trap.
!

!SWA014TrapUpgrade methodsFor: 'starting up' stamp: 'TZ 2/2/2017 21:12'!
activateUpgrade

 	self spawnTrap.
			! !

!SWA014TrapUpgrade methodsFor: 'constants' stamp: 'TZ 2/10/2017 19:47'!
imageFileName
	
	^ 'TrapUpgrade.png'.! !

!SWA014TrapUpgrade methodsFor: 'world manipulation' stamp: 'MV 2/10/2017 17:48'!
offsetToUpgradeOwner
	
	^ self upgradeOwner direction * self upgradeOwner bodysize negated.! !

!SWA014TrapUpgrade methodsFor: 'world manipulation' stamp: 'MV 2/10/2017 17:49'!
spawnTrap
	|aTrap|
	
	aTrap := SWA014Trap new.
	
	aTrap
		center: self upgradeOwner center + self offsetToUpgradeOwner;
		realCenter: self upgradeOwner realCenter + self offsetToUpgradeOwner;
		listOfLivingThings: self upgradeOwner listOfLivingThings;
		yourself.
	
	self upgradeOwner swaworld addMorph: aTrap.! !

!SWA014Upgrade methodsFor: 'starting up' stamp: 'MV 2/10/2017 18:57'!
activateIfNotCooling

	self cooling ifFalse: [
		self
			activateUpgrade;
			cooling: true;
			futureDo: #cooling: at: (self cooldown) args: (Array with: false);
			yourself.
	
		self coolDownMorph
			show;
			futureDo: #hide at: self cooldown args: #();
			yourself].! !

!SWA014Upgrade methodsFor: 'starting up' stamp: 'MV 2/5/2017 21:45'!
activateUpgrade
	"The different Upgrades should overwrite this. This method gets called, when the player wants to use an Upgrade"
	"To add a new Upgrade change: Game->initializeDictionary, Game->handleKeystroke, and right now Game->showEvolveOptions"
	
	self SubclassResponsibility.
	! !

!SWA014Upgrade methodsFor: 'accessing' stamp: 'TZ 2/2/2017 20:46'!
coolDownMorph

	^ coolDownMorph.! !

!SWA014Upgrade methodsFor: 'accessing' stamp: 'TZ 2/2/2017 20:46'!
coolDownMorph: aMorph

	coolDownMorph := aMorph.! !

!SWA014Upgrade methodsFor: 'accessing' stamp: 'MV 2/10/2017 19:15'!
cooldown

	^ cooldown ifNil: [10000].! !

!SWA014Upgrade methodsFor: 'accessing' stamp: 'TZ 2/2/2017 20:46'!
cooldown: aTime

	cooldown := aTime.! !

!SWA014Upgrade methodsFor: 'accessing' stamp: 'MV 2/10/2017 19:13'!
cooling

	^ cooling ifNil: [false].! !

!SWA014Upgrade methodsFor: 'accessing' stamp: 'TZ 2/2/2017 20:46'!
cooling: aBoolean

	cooling := aBoolean.! !

!SWA014Upgrade methodsFor: 'constants' stamp: 'TZ 2/10/2017 19:40'!
imageFileName

	self SubclassResponsibility.! !

!SWA014Upgrade methodsFor: 'initialization' stamp: 'MV 2/10/2017 20:09'!
initialize
	super initialize.
	
	self
		coolDownMorph: ImageMorph new; 
		addMorph: self coolDownMorph;
		image: (Form fromFileNamed: self imageFileName);
		yourself.
		
	self coolDownMorph
		image: (Form fromFileNamed: 'Cooldown.png');
		hide;
		yourself.! !

!SWA014Upgrade methodsFor: 'accessing' stamp: 'TZ 2/2/2017 20:46'!
upgradeOwner

	^ upgradeOwner.! !

!SWA014Upgrade methodsFor: 'accessing' stamp: 'TZ 2/2/2017 20:46'!
upgradeOwner: aLivingThing

	upgradeOwner := aLivingThing.! !

SimpleButtonMorph subclass: #SWA014Button
	instanceVariableNames: 'action'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWA014'!

!SWA014Button commentStamp: 'Tc 2/5/2017 14:17' prior: 0!
I am the EndButton in the Menu. If you click on me, you will end the game.!

!SWA014Button class methodsFor: 'as yet unclassified' stamp: 'MV 2/10/2017 16:26'!
onClick: aBlock

	^ self new
		action: aBlock;
		yourself.! !

!SWA014Button methodsFor: 'accessing' stamp: 'MV 2/10/2017 16:26'!
action

	^ action.! !

!SWA014Button methodsFor: 'accessing' stamp: 'MV 2/10/2017 16:26'!
action: anAction

	action := anAction.! !

!SWA014Button methodsFor: 'initialization' stamp: 'MV 2/10/2017 16:36'!
initialize 
	super initialize.

	self 
		label: 'No Label';
		color: Color red;
		yourself.! !

!SWA014Button methodsFor: 'event handling' stamp: 'MV 2/10/2017 16:27'!
mouseDown: anEvent

	self action value.! !

!Random methodsFor: '*SWA014' stamp: 'MV 2/10/2017 14:04'!
isTrueIn: aNumber outOf: aNumberOfCases
	
	^ aNumberOfCases atRandom <= aNumber.! !

StringMorph subclass: #SWA014Score
	instanceVariableNames: 'scoreValue'
	classVariableNames: 'Highscore'
	poolDictionaries: ''
	category: 'SWA014'!

!SWA014Score commentStamp: 'MV 2/5/2017 21:08' prior: 0!
I am the Score. I will show you, how many points you got so far and if you are good enough, you will beat the highscore, which I hold. I observe the PlayerBlob to give you your points when you eat an Enemy.
!

!SWA014Score class methodsFor: 'accessing' stamp: 'TZ 2/3/2017 18:12'!
highscore
	
	 ^ Highscore.! !

!SWA014Score class methodsFor: 'accessing' stamp: 'TZ 2/3/2017 18:10'!
highscore: aScore
	
	Highscore := aScore.! !

!SWA014Score methodsFor: 'constants' stamp: 'MV 2/10/2017 18:31'!
fontSize

	^ 60.! !

!SWA014Score methodsFor: 'updating' stamp: 'MV 2/5/2017 22:00'!
increaseScore
	
	self scoreValue: self scoreValue + self scorePerEnemyEaten.
	self acceptValue: self scoreValue.
	self color: self randomColor.
	self updateHighscore.
	! !

!SWA014Score methodsFor: 'initialization' stamp: 'MV 2/10/2017 18:31'!
initialize 
	super initialize.
	
	self
		acceptValue: self scoreValue;
		color: Color red;
		width: 500;
		height: 500;
		fontName: 'BitstreamVeraSans' size: self fontSize;
		initializeHighscoreIfNecessary;
		yourself.! !

!SWA014Score methodsFor: 'initialization' stamp: 'MV 2/10/2017 18:31'!
initializeHighscoreIfNecessary

	SWA014Score highscore ifNil: [SWA014Score highscore: 0].! !

!SWA014Score methodsFor: 'accessing' stamp: 'TZ 2/3/2017 18:17'!
randomColor
	
	^ {Color green. Color red. Color yellow. Color blue. Color orange. Color lavender. Color black} atRandom.! !

!SWA014Score methodsFor: 'constants' stamp: 'MV 2/5/2017 22:00'!
scorePerEnemyEaten

	^ 10.! !

!SWA014Score methodsFor: 'accessing' stamp: 'MV 2/10/2017 18:30'!
scoreValue

	^ scoreValue ifNil: [0].! !

!SWA014Score methodsFor: 'accessing' stamp: 'MV 2/5/2017 17:21'!
scoreValue: aValue

	scoreValue := aValue.! !

!SWA014Score methodsFor: 'updating' stamp: 'TZ 2/10/2017 21:34'!
update: aKey

	aKey caseOf: {
		[#playerGrew]->[self increaseScore]} otherwise: [].! !

!SWA014Score methodsFor: 'updating' stamp: 'TZ 2/3/2017 18:21'!
updateHighscore
	
	SWA014Score highscore: (SWA014Score highscore max: self scoreValue). ! !

!Point methodsFor: '*SWA014' stamp: 'MV 2/10/2017 16:21'!
atRandom

	^ (self x atRandom) @ (self y atRandom).! !

!Point methodsFor: '*SWA014' stamp: 'MV 2/4/2017 18:37'!
normalizedWithoutZeroDivide
	^ self = 0 asPoint
		ifTrue: [0]
		ifFalse: [self normalized].
	! !
SWA014ServiceProvider initialize!
